@@ -11,14 +11,19 @@ to the the server.
 
 import sys
 import json
-from functools import wraps
+
+from multiprocessing.dummy import Pool as ThreadPool
+from functools import wraps, partial
 
 try:
     from urllib.request import urlopen, Request, quote
     from urllib.parse import urlencode
+    from queue import Queue
 except ImportError:
     from urllib2 import urlopen, Request, quote
     from urllib import urlencode
+    from Queue import Queue
+
 
 __all__ = [
     'push_url',
@@ -32,11 +37,16 @@ __all__ = [
 
 def push_url(site):
     '''
-    Decorator that wraps the translator method and performs
-    the GET HTTP request.
+    Decorates a function returning the url of translation API.
+    Creates and maintains HTTP connection state
 
     Returns a dict response object from the server containing the translated
     text and metadata of the request body
+
+    :param site: translator
+    :type site: function
+    :return: HTTP Response
+    :rtype: json
     '''
 
     @wraps(site)
@@ -67,7 +77,7 @@ def translator(source, target, phrase):
     :type target: str
     :param phrase: Text body string that will be url encoded and translated
     :type phrase: str
-    :returns: url
+    :return: url
     :rtype: str
 
     List of acceptable language codes for source and target languages
@@ -79,7 +89,7 @@ def translator(source, target, phrase):
     .. code-block:: python
 
         >>> import translate
-        >>> translator('en', 'zh-TW', 'Hello World!')
+        >>> translate.translator('en', 'zh-TW', 'Hello World!')
             '你好世界！'
 
     """
@@ -117,16 +127,28 @@ def coroutine(func):
 
 @coroutine
 def text_sink(source, dest):
-    """Coroutine end-point. Outputs text stream into translator"""
-    while True:
-        line = (yield)
-        translation = translator(source, dest, line)['sentences']
-        for line in translation:
-            sys.stdout.write(line['trans'])
+    """
+    Coroutine end-point. Outputs text stream into translator
+    """
+    task_queue = []
+    pool = ThreadPool(8)
+    try:
+        while True:
+            line = (yield)
+            task_queue.append(line)
+    finally:
+        result = pool.map(partial(translator, source, dest), task_queue)
+
+        pool.close()
+        pool.join()
+
+        for trans in result:
+            for line in trans['sentences']:
+                sys.stdout.write(line['trans'])
+        sys.stdout.write('\n')
+
 
 
-# Make less http requests by chunking.
-# Google maxes chunks sizes around 2k characters
 @coroutine
 def spooler(iterable):
     """Consumes text stream and spools into larger chunk for processing"""
@@ -154,4 +176,3 @@ def source(target):
 
     for line in sys.stdin:
         target.send(line)
-    target.close()
